#!/usr/bin/python
# coding: utf-8


import os
from time import sleep
from random import uniform

class Partie:
    
    #initialisateur d'objet
    def __init__(self,gpio,bouton_start,led):
        self._gpio = gpio
        self._bouton_start = bouton_start
        self._led = led
        self._joueurs = [] 
        self._round = 1
        self._regle = None
        
        self._gpio.setup(self._bouton_start, self._gpio.IN, pull_up_down=self._gpio.PUD_UP) 
        self._gpio.setup(self._led,self._gpio.OUT)
        os.system('clear')
        print ("*************************" \
        "\r\n*        FASTMAN        *" \
        "\r\n*************************" \
        "\r\n* the world fastest man *" \
        "\r\n*************************")

    #ajoute joueur à la partie
    def add_joueur(self,joueur):
        self._joueurs.append(joueur)
        print(joueur.nom + " Est entré dans la partie")

    #défini la regle du jeu
    def set_regle(self,regle):
        self._regle = regle
        print(self._regle.regle)
    
    #démarre la partie
    def start(self):
        if len(self._joueurs) < 2:
             print ("\r\nErreur : Il faut au moins 2 joueurs pour faire une partie !")
             return;
        if self._regle == None:
            print ("\r\nErreur : Aucune règle de jeu n'est définie !")
            return;
        self._partie()

    #deroulement de la partie
    def _partie(self):
        t = 0
        tt = uniform(2.,10.)        
        if self._regle.winner(self._joueurs) or self._regle.looser(self._joueurs):
            self._new_partie()                    
        self._wait_start()
        for j in self._joueurs:
            j.add_bouton_event(self._joueur_event)
        c = self._round
        while self._round == c: 
            t += 0.1
            self._gpio.output(self._led, t <= tt)
            sleep(0.1)
        self._partie()

    #attend le bouton de start pour demarrer le round
    def _wait_start(self):
        print ("\r\nAppuyer sur start pour demarrer le round " + str(self._round))
        while self._gpio.input(self._bouton_start):            
            self._gpio.output(self._led, not self._gpio.input(self._led))
            sleep(0.25) 
        self._regle.scores(self._joueurs,self._round)
        self._gpio.output(self._led,True)

    #evenement declanché par l'appuis de boutons de jouers
    def _joueur_event(self,input):
        win = not self._gpio.input(self._led)
        for j in self._joueurs: 
            j.rem_bouton_event()
        j = self._joueurs[[ x.bouton for x in self._joueurs].index(input)] 
        self._regle.comptage(j,win)
        self._regle.scores(self._joueurs,self._round)
        self._round += 1

    #reinitialisation des score pour nouvelle partie
    def _new_partie(self):
        for j in self._joueurs:
            j.score = 0
            j.triche = 0
        self._round = 1
        print ("\r\nAppuyer sur start pour demarrer une nouvelle partie")
        while self._gpio.input(self._bouton_start): 
            sleep(0.25)
        while not self._gpio.input(self._bouton_start):
            sleep(0.25)
        os.system('clear')


class regleA:
    
    #initialisateur d'objet
    def __init__(self):        
        self._maxscore = 3
        self._maxtriche = 3
        self._regle = "\r\nRegle A" \
        "\r\nChaque coup gagnant incremente le score du joueur" \
        "\r\nChaque coup perdant incremente la triche du joueur" \
        "\r\nle premier joueur arrivant à un score de " + str(self._maxscore) + " gagne" \
        "\r\nou" \
        "\r\nle premier joueur arrivant à une triche de " + str(self._maxtriche) + " perd\r\n" \
     
    #accesseur de self._regle
    def get_regle(self) : return self._regle

    #redefini le scores du joueur suivant res
    def comptage(self,joueur,res):
        if res:
            joueur.score += 1
        else:
            joueur.triche += 1
    
    #affiche le tableau de score
    def scores(self,joueurs,round):
        os.system('clear')
        print ("Round " + str(round))
        for j in joueurs:
            print (j.nom + "  Score:" + str(j.score) + "  Triche:" +  str(j.triche))

    #Retourne Vrai si il y a un gagnant
    def winner(self,joueurs):
        for j in joueurs:
            if j.score == self._maxscore:
                print (j.nom + " a gagné !")
                return True
        return False
        
    #Retourne Vrai si il y a un perdant
    def looser(self,joueurs):
        for j in joueurs:
            if j.triche == self._maxtriche:
                print (j.nom + " a perdu !")
                return True
        return False

    #propriètè regle
    regle = property (get_regle)

class regleB:

    #initialisateur d'objet
    def __init__(self):
        self._maxscore = 3
        self._regle = "Regle B" \
        "\r\nChaque coup gagnant incremente le score du joueur" \
        "\r\nChaque coup perdant décremente le score du joueur" \
        "\r\nle premier joueur arrivant à un score de " + str(self._maxscore) + " gagne\r\n" \

    #accesseur de self._regle
    def get_regle(self) : return self._regle

    #redefini le scores du joueur suivant res
    def comptage(self,joueur,res):
        if res:
            joueur.score += 1
        else:
            joueur.score -= 1
    
    #affiche le tableau de score
    def scores(self,joueurs,round):
        os.system('clear')
        print ("Round " + str(round))
        for j in joueurs:
            print (j.nom + "  Score:" + str(j.score))

    #Retourne Vrai si il y a un gagnant
    def winner(self,joueurs):
        for j in joueurs:
            if j.score == self._maxscore:
                print (j.nom + " a gagné !")
                return True

    #retourne toujours Faux puisque la régle ne définie pas de perdant
    def looser(self,joueurs):
        return False
    
    #propriètè regle
    regle = property (get_regle)
    
        
